:i	static char d_dup2_sccs_id[] = "%W% %G%";
:1#include "D_s.h"
/*
* Name
*   dup, dup2 - duplicate an open file descriptor
*
* Syntax
*   newd = dup(oldd)
*   int newd, oldd;
*
*   dup2(oldd, newd)
*   int oldd, newd;
*
* Description
*   The dup system call duplicates an existing object descriptor.  The
*   argument  oldd  is  a  small  non-negative  integer  index  in the
*   per-process descriptor table. The value must be less than the size
*   of  the  table,  which  is  returned  by  getdtablesize.   The new
*   descriptor, newd, returned by the  call  is  the  lowest  numbered
*   descriptor that is not currently in use by the process.
*
*   The  object  referenced  by  the  descriptor  does not distinguish
*   between references using oldd and newd in any way.  Thus, if  newd
*   and  oldd  are  duplicate references to an open file, read, write,
*   and lseek calls all move a single pointer into  the  file.   If  a
*   separate  pointer  into  the  file  is desired, a different object
*   reference to the file must be obtained by  issuing  an  additional
*   open call.
*
*   In  the second form of the call, specify the value of newd needed.
*   If this descriptor is already in  use,  the  descriptor  is  first
*   deallocated as if a close call had been done.
*
* Return Values
*   The value -1 is returned if an error occurs in either  call.   The
*   external variable errno indicates the cause of the error.
*
* Diagnostics
*   The dup and dup2 system calls fail under the following conditions:
*
*   [EBADF]        The oldd or newd is not a valid active descriptor.
*
*   [EMFILE]       Too many descriptors are active.
*
*   [EINTR]        The dup() or dup2() function was terminated prematurely
*                  by a signal.
*
* See Also
*   accept(2), close(2), getdtablesize(2), lseek(2), open(2), pipe(2),
*   read(2), socket(2), socketpair(2), write(2)
*/

d_dup2(old,new)
{	int   v;
	File *fp;
	Fenter("d_dup2");
:5	V5s "dup2(old=%d,new=%d)",old,new V;
	errno = 0;
	v = dup2(old,new);
:5	V5s "dup2(old=%d,new=%d)=%d [Err %d=%s=%s]",old,new,v,Errinfo V;
	if (fp = d_openfile(old))
		d_newfile(fp->flgs,new,fp->name.v,0);
	Fexit;
	return v;
}
